@@ -31,6 +31,11 @@ class ListField(WritableField):
         super(ListField, self).__init__(*args, **kwargs)
         self.item_field = item_field
 
+    def initialize(self, parent, field_name):
+        super(ListField, self).initialize(parent, field_name)
+        if self.item_field:
+            self.item_field.initialize(parent, field_name)
+
     def to_native(self, obj):
         if self.item_field and obj:
             return [
@@ -57,7 +62,6 @@ class ListField(WritableField):
             errors = {}
             for index, item in enumerate(value):
                 try:
-                    self.item_field.run_validators(item)
                     self.item_field.validate(item)
                 except ValidationError as e:
                     errors[index] = e.messages
@@ -65,8 +69,23 @@ class ListField(WritableField):
             if errors:
                 raise NestedValidationError(errors)
 
+    def run_validators(self, value):
+        super(ListField, self).run_validators(value)
+
+        if self.item_field:
+            errors = {}
+            for index, item in enumerate(value):
+                try:
+                    self.item_field.run_validators(item)
+                except ValidationError as e:
+                    errors[index] = e.messages
+
+            if errors:
+                raise NestedValidationError(errors)
+
+
     def validate_is_list(self, value):
-        if not isinstance(value, list):
+        if value is not None and not isinstance(value, list):
             raise ValidationError(
                 self.error_messages['invalid_type'],
                 code='invalid_type',
@@ -85,6 +104,12 @@ class ListOrItemField(WritableField):
         self.item_field = item_field
         self.list_field = ListField(item_field)
 
+    def initialize(self, parent, field_name):
+        super(ListOrItemField, self).initialize(parent, field_name)
+        if self.item_field:
+            self.item_field.initialize(parent, field_name)
+        self.list_field.initialize(parent, field_name)
+
     def to_native(self, obj):
         if isinstance(obj, list):
             return self.list_field.to_native(obj)
@@ -99,14 +124,28 @@ class ListOrItemField(WritableField):
             return self.item_field.from_native(data)
         return data
 
+    def field_from_native(self, data, files, field_name, into):
+        if isinstance(data, list):
+            return self.list_field.field_from_native(data, files, field_name, into)
+        elif self.item_field:
+            return self.item_field.field_from_native(data, files, field_name, into)
+        else:
+            super(ListOrItemField, self).field_from_native(data, files, field_name, into)
+
     def validate(self, value):
         super(ListOrItemField, self).validate(value)
         if isinstance(value, list):
             self.list_field.validate(value)
         elif self.item_field:
-            self.item_field.run_validators(value)
             self.item_field.validate(value)
 
+    def run_validators(self, value):
+        super(ListOrItemField, self).run_validators(value)
+        if isinstance(value, list):
+            self.list_field.run_validators(value)
+        elif self.item_field:
+            self.item_field.run_validators(value)
+
 
 class DictField(WritableField):
     """
@@ -125,6 +164,11 @@ class DictField(WritableField):
         self.value_field = value_field
         self.unicode_options = unicode_options or self.default_unicode_options
 
+    def initialize(self, parent, field_name):
+        super(DictField, self).initialize(parent, field_name)
+        if self.value_field:
+            self.value_field.initialize(parent, field_name)
+
     def to_native(self, obj):
         if self.value_field and obj:
             return dict(
@@ -151,7 +195,6 @@ class DictField(WritableField):
             errors = {}
             for k, v in six.iteritems(value):
                 try:
-                    self.value_field.run_validators(v)
                     self.value_field.validate(v)
                 except ValidationError as e:
                     errors[k] = e.messages
@@ -159,8 +202,22 @@ class DictField(WritableField):
             if errors:
                 raise NestedValidationError(errors)
 
+    def run_validators(self, value):
+        super(DictField, self).run_validators(value)
+
+        if self.value_field:
+            errors = {}
+            for k, v in six.iteritems(value):
+                try:
+                    self.value_field.run_validators(v)
+                except ValidationError as e:
+                    errors[k] = e.messages
+
+            if errors:
+                raise NestedValidationError(errors)
+
     def validate_is_dict(self, value):
-        if not isinstance(value, dict):
+        if value is not None and not isinstance(value, dict):
             raise ValidationError(
                 self.error_messages['invalid_type'],
                 code='invalid_type',
@@ -188,6 +245,9 @@ class PartialDictField(DictField):
     def validate(self, value):
         super(PartialDictField, self).validate(self._filter_dict(value))
 
+    def run_validators(self, value):
+        super(PartialDictField, self).run_validators(self._filter_dict(value))
+
     def _filter_dict(self, value):
         if isinstance(value, dict):
             return dict(
